#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2023 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>


from gimpfu import *
#from array import array
#import math
#import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)

#####
##### TEMP VALUES

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
darkestgrey = gimpcolor.RGB(20, 20, 20)
verydarkgrey = gimpcolor.RGB(50, 50, 50)
darkgrey = gimpcolor.RGB(75, 75, 75)
midgrey2 = gimpcolor.RGB(105, 105, 105)
midgrey = gimpcolor.RGB(125, 125, 125)
lightgrey = gimpcolor.RGB(170, 170, 170)
verylightgrey = gimpcolor.RGB(222, 222, 222)
red = gimpcolor.RGB(220,10,10)	

# Some global metallic colours we may need for creating gradients

lightsilver = gimpcolor.RGB(204, 202, 202)
darksilver = gimpcolor.RGB(46, 43, 43)
lightgold = gimpcolor.RGB(250, 233, 49)
darkgold = gimpcolor.RGB(240, 183, 42)
lightcopper = gimpcolor.RGB(212, 125, 1)
darkcopper = gimpcolor.RGB(177, 106, 58)
lightbrass = gimpcolor.RGB(253, 242, 91)
darkbrass = gimpcolor.RGB(220 , 118, 20)
lightbronze = gimpcolor.RGB(104, 75, 33)
darkbronze = gimpcolor.RGB(83, 69, 42)


def do_stuff(image, layer, new_mode, do_centre, feather, drop_offset_x, drop_offset_y, drop_blur, drop_opacity, drop_colour, drop_relief):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
	
    #####
    ##### TEMP VALUES
    do_metal = False
    metal_texture = 6
    metal_colour = 0
    retain_original = False
    texture_opacity = 60
    do_gradient = False
    fill_gradient = 0
    bump_depth = 30
    light_amount = 1.50
    compass = 7
    effect_elevation = 30.0
    do_sheen = False
    apply_lighting = False
    shade_of_grey = 50
    effect_mode_ptr = 26
    artifact_radius = 1.00
    artifact_delta = 150
    final_bump_depth = 0
    apply_to_custom = True
	
	
		
    dummy = 0
    active_layer_id = pdb.gimp_image_get_active_layer(image)
    active_layer = pdb.gimp_image_get_active_layer(image)
	
	
	
    # check for user selection
    current_selection = pdb.gimp_image_get_selection(image)
    is_empty = pdb.gimp_selection_is_empty(image)
	
    if is_empty: #complain
        message_text = "Please create a selection with one of the selection tools and run the filter again."
        old_handler = pdb.gimp_message_get_handler()	
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(message_text)
        pdb.gimp_message_set_handler(old_handler)    
    else: # get to work

        # get user selection information
        ds_is_empty, ds_x1, ds_y1, ds_x2, ds_y2 = pdb.gimp_selection_bounds(image)
        decor_selection_width = (ds_x2 - ds_x1)
        decor_selection_height = (ds_y2 - ds_y1)
        #pdb.gimp_message("decor_selection_width = " + str(decor_selection_width) + " : ds_x1 = " + str(ds_x1)  + " : ds_x2 = " + str(ds_x2))
        decor_selection = pdb.gimp_selection_save(image) # save user selection as channel
        pdb.gimp_item_set_name(decor_selection, "Decor Selection") # give user selection a name
		
        pdb.gimp_selection_none(image) # dismiss user selection
		
        # loop through layers to check for required decor layers
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        layer_names_list = image.layers
        d_loop, decorative_count = 0, 0
        while d_loop<num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[d_loop])
            layer_name = layer_name.decode('utf-8')
            # check to see if the current layer is a custom font character layer
            test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[d_loop], layer_name)
            if not test_for_custom_layer:
                if layer_name<>"space.png" and layer_name<>'" ".png':
                    decorative_count += 1
            d_loop+=1
			
        
        if decorative_count == 0: # add two required layers using existing fg and bg colours
            
            decor_layer1 = pdb.gimp_layer_new(image, image.width, image.height, 1, "Decor Layer 1", 100, 28)
            pdb.gimp_image_insert_layer(image, decor_layer1, None, -1)
            pdb.gimp_drawable_edit_fill(decor_layer1,0)
            
            decor_layer2 = pdb.gimp_layer_new(image, image.width, image.height, 1, "Decor Layer 2", 100, 28)
            pdb.gimp_image_insert_layer(image, decor_layer2, None, -1)
            pdb.gimp_drawable_edit_fill(decor_layer2,1)
			
			
            pdb.gimp_item_set_visible(decor_layer1, False)
            pdb.gimp_item_set_visible(decor_layer2, False)
			
			
        elif decorative_count == 1: # adds one required layer using existing bg colour
            decor_layer2 = pdb.gimp_layer_new(image, image.width, image.height, 1, "Decor Layer 2", 100, 28)
            pdb.gimp_image_insert_layer(image, decor_layer2, None, -1)
            pdb.gimp_drawable_edit_fill(decor_layer2,1)
			
            pdb.gimp_item_set_visible(decor_layer2, False)
			
        # set filter variables to loop through character layers			
        loop_ptr = 1
        first_char_layer_name = "none"
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        loop = num_layers - loop_ptr
        layer_names_list = image.layers
        original_active_layer = image.active_layer
		
        num_multi_layers,other_layers,num_char_layers = 0,0,0
        visi_loop, first_char_layer  = 0, -1
        decorative_layers = ""
        decorative_count = 0		
		
        mode = fn_get_mode(new_mode) 		
	
        while visi_loop < num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            # check to see if the current layer is a custom font character layer
            test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
			
            if test_for_custom_layer:
                num_char_layers += 1
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
                    layer_names_list[visi_loop].set_offsets(0, 0)
            else: # count decor layers and add them to list
                if layer_name<>"space.png" and layer_name<>'" ".png':
                    if decorative_count == 0: 
                        decorative_layers = [layer_names_list[visi_loop]]
                    else:
                        decorative_layers.append(layer_names_list[visi_loop])
                    decorative_count += 1
		            
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False) # turn off visibility
            visi_loop += 1
			
		
        while loop >= 0:
	        # get info about layer we are working on for custom font mode
            layer_id = layer_ids[loop]
            layer_name = layer_names_list[loop]
	
            # cater for unicode characters above normal range
            display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
            display_layer_name = display_layer_name.decode('utf-8')	
	
            # check we have the layer id of the currently active layer
            pdb.gimp_image_set_active_layer(image, layer_name)
            layer_id = pdb.gimp_image_get_active_layer(image)

            # check to see if the current layer is a custom font character layer
            test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
	
	
            if test_for_custom_layer:
                # ensure we have the active character layer and make it visible
                active_layer = pdb.gimp_image_get_active_layer(image)
                pdb.gimp_item_set_visible(active_layer, True)
                active_width = active_layer.width
				
                decor_offset = (image.width-active_layer.width)/2
					
                copy_layer = pdb.gimp_layer_new_from_drawable(decorative_layers[1], image) # make a copy of the 2nd decor layer
                pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
                copy2_layer = pdb.gimp_layer_new_from_drawable(decorative_layers[0], image) # make a copy of the 1st decor layer
                pdb.gimp_image_insert_layer(image, copy2_layer, None, -1)
                pdb.gimp_item_set_visible(copy_layer, True)
                pdb.gimp_item_set_visible(copy2_layer, True)
                pdb.gimp_image_select_item(image, 0, decor_selection) # activate the user selection
                if do_centre>1: pdb.gimp_selection_translate(image, (image.width/2) - (ds_x1 + ((decor_selection_width)/2)), 0) # centre selection
                # add user feather if required
                old_feather = pdb.gimp_context_get_feather()
                if feather>0:
                    pdb.gimp_selection_feather(image, feather)
                success = pdb.gimp_edit_cut(copy2_layer)
                pdb.gimp_selection_none(image)
                if drop_colour>0: # add drop shadow if required
                    copy2_layer = proc_dropshadow(image, copy2_layer, copy2_layer, drop_offset_x, drop_offset_y, drop_blur, drop_opacity, drop_colour, drop_relief)
                copy_layer = pdb.gimp_image_merge_down(image, copy2_layer,0)
					
                if do_centre>0: pdb.gimp_item_transform_translate(active_layer, (image.width - active_layer.width)/2, 0) # centre character layer
					
                pdb.gimp_image_select_item(image, 0, active_layer)
                pdb.gimp_selection_invert(image)
                success = pdb.gimp_edit_cut(copy_layer)
                pdb.gimp_selection_none(image)
                if mode <> 0 and mode <> 28: # any mode except Normal modes
                    pdb.gimp_layer_set_mode(copy_layer,mode)
                    active_layer = pdb.gimp_image_merge_down(image, copy_layer,2)
                else:
                    pdb.gimp_edit_clear(active_layer)
                    active_layer = pdb.gimp_image_merge_down(image, copy_layer,2)
                pdb.gimp_layer_set_offsets(active_layer, 0, 0) # return character layer to image origin
			
                        
                pdb.gimp_selection_none(image)
				
            loop_ptr += 1
            loop = num_layers - loop_ptr
            pdb.gimp_item_set_visible(active_layer, False)
			
        pdb.gimp_image_remove_channel(image, decor_selection) # for 1st decor layer selection
        #if not do_centre: pdb.gimp_image_remove_channel(image, decor2_selection) # for 2nd decor layer selection
		
		
        if first_char_layer_name <> "none": # the character layer to make visible on completion
            first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
            pdb.gimp_item_set_visible(first_char_layer_id, True)
            pdb.gimp_image_set_active_layer(image,first_char_layer_id)
        if feather>0: pdb.gimp_context_set_feather(old_feather)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------
# -------------------------------------------------------------------------------------

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################

######################################################################################################################################

def fn_get_mode(new_mode):
        temp_mode = 0
        if new_mode <> 0:
            if new_mode == 1:
                temp_mode = 1
            elif new_mode == 2:
                temp_mode = 57
            elif new_mode == 3:
                temp_mode = 59
            elif new_mode == 4:
                temp_mode = 60
            elif new_mode == 5:
                temp_mode = 36
            elif new_mode == 6:
                temp_mode = 55
            elif new_mode == 7:
                temp_mode = 31
            elif new_mode == 8:
                temp_mode = 42
            elif new_mode == 9:
                temp_mode = 33
            elif new_mode == 10:
                temp_mode = 35
            elif new_mode == 11:
                temp_mode = 54
            elif new_mode == 12:
                temp_mode = 30
            elif new_mode == 13:
                temp_mode = 43
            elif new_mode == 14:
                temp_mode = 53
            elif new_mode == 15:
                temp_mode = 23
            elif new_mode == 16:
                temp_mode = 45
            elif new_mode == 17:
                temp_mode = 44
            elif new_mode == 18:
                temp_mode = 48
            elif new_mode == 19:
                temp_mode = 49
            elif new_mode == 20:
                temp_mode = 50
            elif new_mode == 21:
                temp_mode = 51
            elif new_mode == 22:
                temp_mode = 32
            elif new_mode == 23:
                temp_mode = 52
            elif new_mode == 24:
                temp_mode = 34
            elif new_mode == 25:
                temp_mode = 46
            elif new_mode == 26:
                temp_mode = 47
            elif new_mode == 27:
                temp_mode = 41
            elif new_mode == 28:
                temp_mode = 37
            elif new_mode == 29:
                temp_mode = 38
            elif new_mode == 30:
                temp_mode = 39
            elif new_mode == 31:
                temp_mode = 40
            elif new_mode == 32:
                temp_mode = 24
            elif new_mode == 33:
                temp_mode = 25
            elif new_mode == 34:
                temp_mode = 26
            elif new_mode == 35:
                temp_mode = 27
            elif new_mode == 36:
                temp_mode = 56
        mode = temp_mode
			

        return mode
	
	
######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################
		
def proc_dropshadow(im, original_layer, target_layer, drop_offset_x, drop_offset_y, drop_blur, drop_opacity, drop_colour, drop_relief):
    pdb.gimp_image_freeze_layers(im)
    # make a copy of the image width/height
    old_im_width, old_im_height = im.width, im.height
    name = pdb.gimp_item_get_name(target_layer)
    # Copy layer to use as the drop shadow
    shadow_layer = pdb.gimp_layer_copy(target_layer, False)
    # Add and re-position the layer in the layer stack
    im.add_layer(shadow_layer, -1)
    pdb.gimp_image_lower_item(im, shadow_layer)
    # get layer offsets for the shadow layer (which will also be the same as the original layer holding the user text)
    shadow_offset_x, shadow_offset_y = pdb.gimp_drawable_offsets(shadow_layer)
    # work out the centre point of the shadow/original layer
    shad_centre_x = (shadow_layer.width/2)
    shad_centre_y = (shadow_layer.height/2)
    # make up new size for the shadow layer to accommodate blurred result - currently twice the existing width and height
    new_shad_width = shadow_layer.width*2
    new_shad_height = shadow_layer.height*2
    # work out new offsets for the new size of the shadow layer to ensure that the shadow and original layers are still aligned
    new_shad_layer_offset_x = 0 - ((shadow_layer.width - new_shad_width)/2)
    new_shad_layer_offset_y = 0 - ((shadow_layer.height - new_shad_height)/2)
    # and apply the new offsets to the shadow layer
    pdb.gimp_layer_resize(shadow_layer, new_shad_width, new_shad_height, new_shad_layer_offset_x, new_shad_layer_offset_y)
    # make a note of the new shadow layer offsets
    shadow_resized_offset_x, shadow_resized_offset_y = pdb.gimp_drawable_offsets(shadow_layer)	
	
    # if any of the shadow layer falls outside the image boundaries
    reset_shadow = False
    if new_shad_layer_offset_x<0 or new_shad_layer_offset_y<0 or shadow_layer.width > im.width or shadow_layer.height > im.height:
        reset_shadow = True
        # set offsets of the shadow layer to image origin
        pdb.gimp_layer_set_offsets(shadow_layer,0,0)
        # set offsets for original text layer to match
        pdb.gimp_layer_set_offsets(target_layer,0 + abs(new_shad_layer_offset_x) ,0 + abs(new_shad_layer_offset_y))
        # resize the image to accomodate the new shadow layer size
        new_im_width, new_im_height = im.width, im.height
        if shadow_layer.width > im.width: new_im_width = shadow_layer.width
        if shadow_layer.height > im.height: new_im_height = shadow_layer.height
        pdb.gimp_image_resize(im, new_im_width, new_im_height, 0, 0)
		
	
	
    # get alpha selection
    pdb.gimp_image_select_item(im, 2, shadow_layer)
    if drop_offset_x==0 and drop_offset_y==0 and drop_relief>0: pdb.gimp_selection_grow(im, drop_relief)
    # fill the shadow layer with colour
    if drop_colour<3:
        pdb.gimp_drawable_edit_fill(shadow_layer,drop_colour-1)
    if drop_colour==3:
        old_fg = pdb.gimp_context_get_foreground()
        pdb.gimp_context_set_foreground(black)
        pdb.gimp_drawable_edit_fill(shadow_layer,0)
        pdb.gimp_context_set_foreground(old_fg)
    # deselect
    pdb.gimp_selection_none(im)
    # move the shadow layer to user position and adjust opacity
    temp = pdb.gimp_layer_translate(shadow_layer, drop_offset_x, drop_offset_y)
    #pdb.gimp_image_select_item(im, 2, original_layer)
    # blur shadow 
    pdb.plug_in_gauss(im, shadow_layer, drop_blur, drop_blur, 1)
    #pdb.gimp_selection_none(im)
    # crop the blurred layer
    pdb.plug_in_autocrop_layer(im, shadow_layer)
    # move the shadow layer to user position and adjust opacity
    #temp = pdb.gimp_layer_translate(shadow_layer, drop_offset_x, drop_offset_y)
    pdb.gimp_layer_set_opacity(shadow_layer, drop_opacity)
    # Copy layer to use as the reference
    ref_layer = pdb.gimp_layer_copy(target_layer, False)
    position = pdb.gimp_image_get_item_position(im, shadow_layer)
    # Add and re-position the layer in the layer stack
    im.add_layer(ref_layer, position+1)
    # merge and rename
    target_layer = pdb.gimp_image_merge_down(im,target_layer,0)
    target_layer = pdb.gimp_image_merge_down(im,target_layer,2)
    pdb.gimp_item_set_name(target_layer, name)
    if reset_shadow: 
        cropped_offset_x, cropped_offset_y = pdb.gimp_drawable_offsets(target_layer)	
        pdb.gimp_layer_set_offsets(target_layer,shadow_offset_x, shadow_offset_y)
    pdb.gimp_image_resize(im, old_im_width, old_im_height, 0, 0)
    
    pdb.gimp_image_thaw_layers(im)
    return target_layer
 	
#####################################################################################################

    
######################################################################################################################################

def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away
	
######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Apply_Selection_v0_1",
    "Provides options for using selections with custom fonts",
    "Provides options for using selections with custom fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2023",
    "<Image>/Filters/Custom Fonts/Decorating Tools/Custom Font Apply Selection v0.1...",
    "RGB*",
    [
    (PF_OPTION, "mode_option2", "New Mode to use with selection: ", 0, ("Normal", "Dissolve","Color erase", "Merge", "Split", "Lighten only", "Luma/Luminance lighten only", "Screen", "Dodge", "Addition", "Darken only", "Luma/Luminance darken only", "Multiply", "Burn", "Linear burn", "Overlay", "Soft light", "Hard light", "Vivid light", "Pin light", "Linear light", "Hard mix", "Difference", "Exclusion", "Subtract", "Grain extract", "Grain merge", "Divide", "Hue (HSV)", "Saturation (HSV)", "Color (HSL)", "Value (HSL)", "Hue (LCH)", "Chroma (LCH)", "Color (LCH)", "Lightness (LCH)", "Luminance")),
    (PF_OPTION, "usecentreoption", "Apply horizontal centring: ", 0, ("None", "To characters only", "To characters and the selection")),
    (PF_SPINNER, "feathersize", "Feather selection by: ", 0, (0, 500, 1)),
    (PF_SLIDER, "pf_drop_offset_x", "Drop shadow offset x position: ", 8, (-50, 50, 1)),
		
    (PF_SLIDER, "pf_drop_offset_y", "Drop shadow offset y position: ", 8, (-50, 50, 1)),
    (PF_SLIDER, "pf_drop_blur", "Drop shadow blur percentage: ", 15, (0, 50, 1)),
    (PF_SLIDER, "pf_drop_opacity", "Drop shadow opacity: ", 70, (20, 100, 1),{'newLine':'1','tab':1}),
    (PF_OPTION, "drop_colour", "Drop shadow type", 0, ( "No Shadow", "Foreground Colour", "Background colour", "Black")),
    (PF_SPINNER, "pf_relief", "Drop shadow relief: ", 8,(0,24,1)),
    ],
    [],
    do_stuff)

main()
